require "postjob/cli"
require "postjob/migrations"

module Postjob::CLI
  # Prints all settings in the queue
  def db_settings
    connect_to_database!

    raise "No settings in this database schema" unless ::Postjob::Queue.settings?

    Simple::SQL.print "SELECT name,value FROM postjob.settings ORDER BY name"
  end

  def db_migrate
    connect_to_database!
    Postjob::Migrations.migrate!
  end

  def db_unmigrate
    unless force
      confirm! <<~TXT
        Really unmigrating database? This will destroy all data about postjobs!
        (To prevent the need to confirm run with '--force'.)
      TXT
    end

    connect_to_database!
    Postjob::Migrations.unmigrate!
  end

  def db_remigrate(force: false)
    unless force
      confirm! <<~TXT
        Really remigrating database? This will destroy all data about postjobs!
        (To prevent the need to confirm run with '--force'.)
      TXT
    end

    connect_to_database!
    Postjob::Migrations.unmigrate!
    Postjob::Migrations.migrate!
  end

  def confirm!(msg)
    STDERR.puts <<~TXT
      #{msg.chomp}

      Press return to continue, ^C to cancel...
    TXT
    STDIN.gets
  rescue Interrupt
    raise "Cancelled by user"
  end

  private

  USE_ACTIVE_RECORD = false

  def connect_to_database!
    return if @connected_to_database

    @connected_to_database = true
    if USE_ACTIVE_RECORD
      require "active_record"
      abc = ::Simple::SQL::Config.read_database_yml
      ::ActiveRecord::Base.establish_connection abc
    else
      ::Simple::SQL.connect!
    end
  end
end
