# rubocop:disable Lint/HandleExceptions
# rubocop:disable Metrics/MethodLength

module Postjob::CLI
  private

  def events_query(limit:)
    limit = Integer(limit)

    sql = <<-SQL
      SELECT
        events.id,
        events.name,
        events.postjob_id AS job_id,
        postjobs.workflow
          || (CASE WHEN postjobs.workflow_version != '' THEN '@' ELSE '' END)
          || postjobs.workflow_version
          || (CASE WHEN postjobs.workflow_method != 'run' THEN '.' || postjobs.workflow_method ELSE '' END)
          || postjobs.args AS job,
          worker_session_id,
        events.created_at
      FROM postjob.events events
      LEFT JOIN postjob.postjobs postjobs ON events.postjob_id=postjobs.id
      WHERE events.name != 'heartbeat'
    SQL

    scope = Simple::SQL::Scope.new(sql)
    scope
      .order_by("events.id DESC")
      .paginate(per: limit, page: 1)
  end

  public

  # Show the latest job event
  #
  # Example:
  #
  #   postjob events
  def events(limit: "30")
    expect! limit => /\A\d+\z/
    limit = Integer(limit)

    connect_to_database!

    query = events_query(limit: limit)

    print_results query: query
  end

  # Show up-to-date events information once per second
  def events_top(limit: "30")
    loop do
      system "clear"
      events(limit: limit)
      sleep 1
    end
  rescue Interrupt
  end
end
