# rubocop:disable Lint/HandleExceptions

module Postjob::CLI
  private

  def heartbeat_query(limit:)
    limit = Integer(limit)

    sql = <<-SQL
      SELECT
        name,
        postjob_id AS job_id,
        events.host_id,
        (events.attributes->>'uptime')::interval AS uptime,
        to_char((events.attributes->>'cpu_load_1min')::float, '99D99') AS cpu_load,
        events.attributes->>'net_in_1min' AS net_in,
        events.attributes->>'net_out_1min' AS net_out,
        events.attributes->>'net_errors_1min' AS net_errors,
        now() at time zone 'utc' - events.created_at AS age
      FROM postjob.events events
      LEFT JOIN postjob.worker_sessions worker_sessions ON events.worker_session_id=worker_sessions.id
      WHERE events.name = 'heartbeat'
    SQL

    scope = Simple::SQL::Scope.new(sql)
    scope
      .order_by("events.id DESC")
      .paginate(per: limit, page: 1)
  end

  public

  # Show the latest heartbeat events
  def heartbeat(limit: "30")
    expect! limit => /\A\d+\z/
    limit = Integer(limit)

    connect_to_database!

    query = heartbeat_query(limit: limit)

    Postjob.logger.info "CPU load and friends are for the last minute"
    print_results query: query
  end

  # Show up-to-date heartbeat information once per second
  def heartbeat_top(limit: "30")
    loop do
      system "clear"
      heartbeat(limit: limit)
      sleep 1
    end
  rescue Interrupt
  end
end
