module Postjob::CLI
  # Resolve a job by token
  def job_resolve(token, result)
    connect_to_database!
    Postjob.resolve(token: token, result: result)
  end

  def job_enqueue(workflow, *args, queue: nil, tags: nil)
    logger "The job:enqueue command is deprecated, pls use enqueue instead."
    enqueue(workflow, *args, queue: queue, tags: tags)
  end

  # Reset failed jobs
  #
  # This resets all failed jobs within the job tree, below the passed in
  # job id.
  def job_reset(job_id)
    connect_to_database!

    job_id = Integer(job_id)
    full_job_id = Simple::SQL.ask "SELECT full_id FROM postjob.postjobs WHERE id=$1", job_id
    full_job_id || logger.error("No such job: #{job_id}")

    job_ids = Simple::SQL.all <<~SQL
      SELECT id FROM postjob.postjobs
      WHERE (full_id LIKE '#{full_job_id}.%' OR full_id='#{full_job_id}')
        AND status IN ('failed', 'err', 'timeout')
    SQL

    logger.warn "Affected jobs: #{job_ids.count}"
    return if job_ids.empty?

    Simple::SQL.ask <<~SQL, job_ids
      UPDATE postjob.postjobs
      SET
        status='ready', next_run_at=(now() at time zone 'utc'),
        results=null, failed_attempts=0, error=NULL, error_message=NULL, error_backtrace=NULL
      WHERE id = ANY($1);
    SQL

    logger.warn "The following jobs have been reset: #{job_ids.join(', ')}"
  end

  # Kills a specific job
  def job_kill(job_id)
    connect_to_database!

    job_id = Integer(job_id)

    Simple::SQL.ask <<~SQL, job_id
      UPDATE postjob.postjobs
      SET
        status='failed',
        next_run_at=null,
        error='Manually terminated',
        error_message='Manually terminated'
      WHERE id = $1 AND status IN ('ready', 'err', 'sleep')
    SQL
  end

  # runs a job as soon as possible
  def job_force(job_id)
    connect_to_database!

    job_id = Integer(job_id)

    Simple::SQL.ask <<~SQL, job_id
      UPDATE postjob.postjobs
      SET
        next_run_at=now() at time zone 'utc'
      WHERE id = $1 AND status IN ('ready', 'err', 'sleep')
    SQL
  end

  private

  # parses "foo:bar,baz:quibble" into { "foo" => "bar", "baz" => "quibble"}
  def parse_tags(tags)
    return nil unless tags
    tags.split(",").inject({}) do |hsh, tag|
      expect! tag => /\A[^:]+:[^:]+\z/
      k, v = tag.split(":", 2)
      hsh.update k => v
    end
  end
end
