# rubocop:disable Lint/HandleExceptions

module Postjob::CLI
  # Show queue status
  def queues
    connect_to_database!

    Simple::SQL.print <<~SQL
      WITH
        queues AS (
          SELECT DISTINCT(queue) AS queue FROM postjob.postjobs
        ),
        jobs AS (
          SELECT
            queue,
            SUM(IFF(status IN ('ready', 'sleep'), 1, 0))    AS waiting,
            SUM(IFF(status IN ('processing'), 1, 0))        AS processing,
            SUM(IFF(status='ok', 1, 0))                     AS succeeded,
            SUM(IFF(status IN ('failed', 'timeout'), 1, 0)) AS failed
          FROM postjob.postjobs
          GROUP BY queue
        ),
        workflows AS (
          SELECT
            queue,
            SUM(IFF(status IN ('ready', 'sleep'), 1, 0))    AS waiting,
            SUM(IFF(status IN ('processing'), 1, 0))        AS processing,
            SUM(IFF(status='ok', 1, 0))                     AS succeeded,
            SUM(IFF(status IN ('failed', 'timeout'), 1, 0)) AS failed
          FROM postjob.postjobs
          WHERE id=root_id
          GROUP BY queue
        ),
        workers AS (
          SELECT
            sq.queue,
            COUNT(*) AS count
          FROM (
            SELECT
              hosts.id,
              UNNEST(sessions.queues) AS queue,
              hosts.status,
              heartbeats.created_at AS latest_heartbeat
            FROM postjob.hosts hosts
            INNER JOIN postjob.worker_sessions sessions ON sessions.host_id=hosts.id AND sessions.status = 'running'
            LEFT JOIN (
              SELECT host_id, MAX(created_at) AS created_at
              FROM postjob.events events
              WHERE name='heartbeat'
                AND created_at > now() at time zone 'utc' - interval '5 minutes'
              GROUP BY host_id
            ) heartbeats ON hosts.id=heartbeats.host_id
            WHERE hosts.status in ('running', 'shutdown') AND heartbeats.created_at IS NOT NULL
          ) sq
          GROUP BY queue
        )
      SELECT
        queues.queue,
        COALESCE(workers.count, 0)          AS "workers",
        COALESCE(jobs.waiting, 0)           AS "jobs waiting",
        COALESCE(jobs.processing, 0)        AS "jobs processing",
        COALESCE(jobs.succeeded, 0)         AS "jobs succeeded",
        COALESCE(jobs.failed, 0)            AS "jobs failed",
        COALESCE(workflows.waiting, 0)      AS "workflows waiting",
        COALESCE(workflows.processing, 0)   AS "workflows processing",
        COALESCE(workflows.succeeded, 0)    AS "workflows succeeded",
        COALESCE(workflows.failed, 0)       AS "workflows failed"
      FROM queues
      LEFT JOIN workers USING(queue)
      LEFT JOIN jobs USING(queue)
      LEFT JOIN workflows USING(queue)
      ORDER BY queue
    SQL
  end

  # Show up-to-date queue information once per second
  def queues_top
    loop do
      system "clear"
      queues
      sleep 1
    end
  rescue Interrupt
  end
end
