require_relative "./record"

#
# A job
#
class Postjob::Job < Postjob::Record
  def self.find(job_id)
    scope = Postjob::Queue.search("postjobs", id: job_id)
    Simple::SQL.ask(scope, into: Postjob::Job)
  end

  attr_reader :id
  attr_reader :parent_id
  attr_reader :full_id
  attr_reader :root_id
  attr_reader :created_at
  attr_reader :queue
  attr_reader :workflow
  attr_reader :workflow_method
  attr_reader :workflow_version
  attr_reader :args
  attr_reader :next_run_at
  attr_reader :timing_out_at
  attr_reader :failed_attempts
  attr_reader :max_attempts
  attr_reader :status
  attr_reader :results
  attr_reader :error
  attr_reader :error_message
  attr_reader :error_backtrace
  attr_reader :recipients
  attr_reader :timed_out
  attr_reader :tags
  attr_reader :last_worker_session_id
  attr_reader :is_sticky
  attr_reader :sticky_host_id

  STATUSES = %w(ok ready processing sleep err failed timeout)

  def resolve
    expect! status => STATUSES

    case status
    when "ok"       then result
    when "ready"    then :pending
    when "processing" then :pending
    when "sleep"    then :pending
    when "timeout"  then raise Timeout::Error
    when "err"      then :pending
    when "failed"   then raise Postjob::Error::Nonrecoverable, self
    end
  end

  def resolved?
    expect! status => STATUSES
    %w(ok timeout failed).include?(status)
  end

  def result
    results && results.first
  end

  def to_s
    full_workflow = workflow
    full_workflow += "@#{workflow_version}" if workflow_version != ""
    full_workflow += ".#{workflow_method}" if workflow_method != "run"

    args = (self.args || []).map(&:inspect).join(", ")
    "Postjob##{full_id}: #{full_workflow}(#{args}) (#{status})"
  end
end
