# rubocop:disable Security/Eval

module Postjob
  module Migrations
    extend self

    SQL = ::Simple::SQL
    SCHEMA_NAME = ::Postjob::Queue::SCHEMA_NAME
    CHANNEL     = ::Postjob::Queue::Notifications::CHANNEL
    CLIENT_VERSION = "ruby/#{::Postjob::VERSION}"
    DEFAULT_QUEUE = ::Postjob::DEFAULT_QUEUE

    # Note that the SCHEMA_NAME should not be the default name, since unmigrate!
    # below drops that schema, and we don't want to drop the default schema.
    expect! SCHEMA_NAME != "public"

    def unmigrate!
      ::Postjob::Host.clear_storage

      SQL.exec "SET client_min_messages TO WARNING"
      SQL.exec "DROP SCHEMA IF EXISTS #{SCHEMA_NAME} CASCADE"
    end

    def migrate!
      SQL.exec "CREATE SCHEMA IF NOT EXISTS #{SCHEMA_NAME}"

      Dir.glob(__FILE__.gsub(/\.rb$/, "/**/*.{sql,rb}")).sort.each do |file|
        run_migration file
      end
    end

    private

    def run_migration(file)
      Postjob.logger.debug "[postjob] Running migration in #{file}"

      case file
      when /\.rb$/  then run_migration_ruby(file)
      when /\.sql$/ then run_migration_sql(file)
      end
    end

    def run_migration_ruby(file)
      eval File.read(file)
    end

    def run_migration_sql(file)
      sql = File.read(file)
      sql.gsub!(/\{([_A-Za-z0-9+]+)\}/) { |_| const_get(Regexp.last_match(1)) }
      SQL.exec sql
    end
  end
end
