DO $$
  BEGIN
    ALTER TABLE {SCHEMA_NAME}.postjobs ADD COLUMN cron_interval integer;
  EXCEPTION
    WHEN duplicate_column THEN RAISE DEBUG 'column {SCHEMA_NAME}.postjobs.cron_interval already exists';
  END;
$$;

--- define triggers to automatically restart cron jobs ------------------------

-- This method is called whenever a job's status changes. It enqueues a fresh

--
-- An event is created whenever a job's status changes.
--

CREATE OR REPLACE FUNCTION {SCHEMA_NAME}._restart_cron_job()
  RETURNS TRIGGER
AS $$
DECLARE
  p_new_jonb_id BIGINT;
BEGIN
  IF NEW.status = OLD.status THEN
    RETURN NEW;
  END IF;

  IF TG_OP != 'UPDATE' OR NEW.status NOT IN ('failed', 'ok') THEN
    RETURN NEW;
  END IF;

  IF NEW.cron_interval IS NULL THEN
    RETURN NEW;
  END IF;

  SELECT id INTO p_new_jonb_id FROM {SCHEMA_NAME}.enqueue(
    NEW.last_worker_session_id,                                                     -- p_worker_session_id
    NEW.queue,                                                                      -- queue
    NEW.workflow,                                                                   -- workflow
    NEW.workflow_method,                                                            -- workflow_method
    NULL,                                                                           -- workflow_version
    NEW.args,                                                                       -- args
    NULL,                                                                           -- parent_id
    NEW.tags,                                                                       -- tags
    NEW.max_attempts,                                                               -- max_attempts
    (EXTRACT(EPOCH FROM NEW.timing_out_at) - EXTRACT(EPOCH FROM NEW.created_at)),   -- timeout
    NEW.cron_interval,
    NEW.is_sticky,
    NEW.is_greedy
  );

  UPDATE {SCHEMA_NAME}.postjobs
  SET next_run_at = now() at time zone 'utc' + NEW.cron_interval * interval '1 second'
  WHERE id=p_new_jonb_id;

  RETURN NEW;
END;
$$ LANGUAGE plpgsql;

BEGIN;
  DROP TRIGGER IF EXISTS _restart_cron_job ON {SCHEMA_NAME}.postjobs;

  CREATE TRIGGER _restart_cron_job AFTER UPDATE
    ON {SCHEMA_NAME}.postjobs
    FOR EACH ROW
    EXECUTE PROCEDURE {SCHEMA_NAME}._restart_cron_job();
COMMIT;
