# rubocop:disable Lint/RescueException

require_relative "./record"

require "tools/heartbeat"

# A worker worker_session
class Postjob::WorkerSession < Postjob::Record
  class << self
    # Starts a worker session.
    def start!(workflows_with_versions, heartbeat: true, queues:)
      host_id = ::Postjob.host_id
      worker_session = ::Postjob::Queue.worker_session_start(workflows_with_versions, host_id: host_id, queues: queues)

      Postjob.logger.info "Starting worker_session #{worker_session.inspect}, on pid #{$$}"

      start_heartbeat_monitor(host_id) if heartbeat
      worker_session
    end

    def stop!(worker_session)
      ::Postjob::Queue.worker_session_stop(worker_session.id)
    end

    # Starts a heartbeat monitor in the background (i.e. in a new thread).
    def start_heartbeat_monitor(host_id)
      Thread.new do
        begin
          Simple::SQL.connect!
          run_heartbeat_monitor(host_id)
        rescue Exception => e
          STDERR.puts "#{e}, from \n\t#{e.backtrace[0, 5].join("\n\t")}"
        end
      end
    end

    private

    # This message is used during specs.
    def run_heartbeat_monitor(host_id, &block)
      Heartbeat.monitor(60) do |measurement|
        Postjob::Queue.host_heartbeat(host_id, measurement)
        next true unless block
        yield
      end
    end
  end

  attr_reader :id
  attr_reader :host_id
  attr_reader :client_socket
  attr_reader :workflows
  attr_reader :attributes
  attr_reader :created_at
  attr_reader :queues

  def to_s
    "Session##{id}"
  end

  def inspect
    # [TODO] - grepping workflows by /\d$/ to only count workflows with a version number is a bit hackish.
    workflow_count = self.workflows.grep(/\d$/)
    queues         = self.queues.map(&:inspect).join(", ")

    "<Session##{id} w/host_id: #{host_id}, queues: #{queues}, client_socket: #{client_socket}, #{workflow_count} workflows>"
  end
end
