# This module defines the interface available for workflow modules.
#
# This module is extended into the actual workflow module during registering.
module Postjob::Workflow
  # Note: we would like to sure that there is a public run method, and we could
  # inspect the module with a +extended+ callback. This, however, is only
  # possible when the run method is already defined. So, with a job definition
  # of
  #
  #     module Foo
  #       include Postjob::Workflow
  #       def self.run(arg); ..; end
  #     end
  #
  # we cannot run such a test.

  # see Postjob::Runner.async
  def async(runner, *args, timeout: nil, max_attempts: nil, queue: nil)
    ::Postjob::Runner.async(runner, *args, timeout: timeout, max_attempts: max_attempts, queue: queue)
  end

  # see Postjob::Runner.await
  def await(job, *args, timeout: nil, max_attempts: nil)
    ::Postjob::Runner.await(job, *args, timeout: timeout, max_attempts: max_attempts)
  end

  # see Postjob::Runner.workflow_token
  def workflow_token(job)
    expect! job.parent_id => ::Postjob::Runner.current_job.id
    ::Postjob::Queue.find_or_create_token(job)
  end
end
