# A History object, which holds the last +size+ objects in memory.
#
# Implemented as a ring buffer which wraps over.
class History # :nodoc:
  attr_reader :size

  def initialize(size)
    @buffer = Array.new(size)
    @size   = size
    @writer = 0
  end

  # returns the nth-last entry
  def last_nth(idx)
    raise ArgumentError, "RingBuffer size #{@size} is too small to hold #{idx} entries" if idx > @size

    pos = (@writer - idx) % @size
    @buffer[pos]
  end

  def <<(element)
    @writer = (@writer + 1) % @size
    @buffer[@writer] = element
  end
end
